import ns from '@/utils/nsQuery/nsQuery';
import * as store from '@/utils/nsQuery/nsStore';
import {autoForm} from '@/api/component/auto-form';//require auto form data API
import autoFormResCheck from './res-check/methods/res-check';//data check methods
import loginfo from './log';//require auto form data API


let vm, requireData = null;

/**
 * log - descriptor
 * @param type
 * @returns {Function}
 */
let log = (type) => {
  return function (target, name, descriptor) {
    let fomrLog = descriptor.value;
    descriptor.value = (...arg) => {
      fomrLog.apply(target, arg);
      switch (type) {
        case 'remote':
          loginfo.remote(type, vm, requireData);
          break;
        case 'local':
          loginfo.local(type, vm);
          break;
        case 'dataCheck':
          loginfo.dataCheck(vm);
        default:
          break;
      }
    }
  }
}

class dataProject {
  @log('remote')
  /**
   * request data for auto-form  (funcId / interpreter to head, query in params)
   * @param requireData
   * @param cb
   */
  remote(requireData, cb) {
    console.log(requireData)
    autoForm(requireData.url, requireData.head, requireData.query,)
      .then(response => {
        cb(response.resultData);
      }).catch(response => {
      console.log('表单数据请求失败！！');
      return Promise.reject(response);
    });
  }

  @log('local')
  /**
   * request loacl-data
   * @param localdata
   * @param cb
   */
  local(localdata, cb) {
    cb(localdata);
  }

  @log('dataCheck')
  /**
   * check auto-form data
   * @param vm
   * @param data
   * @param isCheck   check data switch
   */
  dataCheck(vm, data, isCheck) {
    if (data && isCheck) {
      //表单各条目数据展示模块
      vm.formDataShow = JSON.parse(JSON.stringify(data));//assign 'formData' to 'formDataShow' by deep copy.
      //错误数据条目列表
      vm.dataCheckResult = autoFormResCheck(data, vm.autoFormID);//output detection results List
      //open table - dialog
      vm.$set(vm.dialogVisible, 'visible', true);
    }
  }
}

let dp = new dataProject()

/**
 * Loading auto-form data
 * @param _t              this vm
 * @constructor
 */
const LoadingData = (_t) => {
  vm = _t;                  //store vm
  vm.loading = true;
  //set autp-form request Id ( 设置 当前自定义表单请求的ID值 )
  vm.$store.dispatch('setRequestFormId', vm.autoFormID);
  //存储当前表单组件的vue实例
  store.vm.set(vm.autoFormID, vm);
  //本地数据和本地数据格式内容符合要求（不为空，''，{}，[]....）
  if (vm.isLocal) {
    if (!ns.base.isEmpty(vm.localData)) {
      dp.local(vm.localData, (localdata) => {
        vm.formData = localdata;
      });
      _decorate(vm);//other handle for auto-form
    }
  }
  else {
    //require data
    requireData = {
      url: vm.requestUrl,
      query: vm.query,
      head: {
        funcId: store.funcId.get(), //funcId
        interpreter: store.formController.get(vm.autoFormID, 'interpreter'),//interpreter
        formOperateType: store.formController.get(vm.autoFormID, 'formOperateType'),//formOperateType
      }
    }
    dp.remote(requireData, (res) => {
      vm.formData = res;
      _decorate(vm);//other handle for auto-form
    });
  }

  /**
   * other handle for auto-form
   * @param vm
   * @private
   */
  function _decorate(vm) {
    //log information after data getted
    loginfo.datagetted(vm.formData);
    console.log('处理前的modelData');
    console.log(vm.formData.modelData);
    //请求数据成功后的外置操作处理数据
    vm.$emit('afterRequest', vm, vm.formData);
    console.log('处理后的modelData');
    console.log(vm.formData.modelData);
    //自动表单初始化 model-data 数据存储
    const initModelData = ns.base.deepClone(vm.formData.modelData);
    vm.$store.dispatch('initModelDataStore', initModelData);
    console.log('存入的modelData值')
    console.log(initModelData)
    //log information after second handle
    loginfo.afterRequest(vm.formData);
    //check data
    dp.dataCheck(vm, vm.formData, vm.isCheck);
    //other auto-form element control switch operation
    vm.itemsRender = true;
    vm.loading = false;
  }
}

export default LoadingData;
